package ex06

import scala.swing.Component
import rescala.events._
import rescala._
import makro.SignalMacro.{ SignalM => Signal }
import java.awt.Point
import scala.swing.event._

/**
 * Utility object providing REScala events and signals for mouse events and mouse state of the given component.
 */
class Mouse(component: Component) {
  /* EScala events */
  val mouseMovedE = new ImperativeEvent[Point]()
  val mousePressedE = new ImperativeEvent[Point]()
  val mouseDraggedE = new ImperativeEvent[Point]()
  val mouseReleasedE = new ImperativeEvent[Point]()

  /* Compose reactive values */
  val mouseChangePosition = mouseMovedE || mouseDraggedE
  val mousePressedOrReleased = mousePressedE || mouseReleasedE
  val position: Signal[Point] = mouseChangePosition.latest(new Point(0, 0))
  val pressed: Signal[Boolean] = mousePressedOrReleased.toggle(Signal { false }, Signal { true }) // TODO: solve this more robust

  /* Scala swing reaction */
  private val react: scala.swing.Reactions.Reaction = {
    case e: MouseMoved => { mouseMovedE(e.point) }
    case e: MousePressed => mousePressedE(e.point)
    case e: MouseDragged => { mouseDraggedE(e.point) }
    case e: MouseReleased => mouseReleasedE(e.point)
  }
  component.listenTo(component.mouse.moves, component.mouse.clicks)
  component.reactions += react
}

