package ex06
package pong

import rescala.events._
import java.awt.Point
import java.awt.Rectangle
import rescala._
import makro.SignalMacro.{ SignalM => Signal }
import java.awt.geom.Area
import java.awt.geom.Ellipse2D

object Pong {
  val Max_X = 800
  val Max_Y = 400
}

object Ball {
  val Size = 20
}

class Pong(val tick: Event[Unit], val mouse: Mouse) {
  import rescala.conversions.SignalConversions._
  val LeftRacketPos = 30
  val RightRacketPos = 770
  val initPosition = new Point(400, 200)
  val speed = new Point(10, 8)

  val x: Signal[Int] = tick.fold(initPosition.x) { (pos, _) => pos + speedX.get}
  val y: Signal[Int] = tick.fold(initPosition.y) { (pos, _) => pos + speedY.get}
  
  val ballArea = Signal { new Area(new Ellipse2D.Double(x(), y(), Ball.Size, Ball.Size)) }

  val mouseY = Signal { mouse.position().getY().toInt }

  val leftRacket = new Racket(LeftRacketPos, mouseY)
  val rightRacket = new Racket(RightRacketPos, y)

  val rackets = List(leftRacket, rightRacket)
  val racketBounds = Signal { rackets.map(_.bounds()) }
  val ballInRacket = Signal { racketBounds().exists{ b => ballArea().intersects(b) }  }
  val collisionRacket = ballInRacket.changedTo(true)

  val leftWall = x.changed && (x => x < 0)
  val rightWall = x.changed && (x => x + Ball.Size > Pong.Max_X)

  val xBounce = leftWall || rightWall || collisionRacket
  val yBounce = y.changed && (y => y < 0 || y + Ball.Size > Pong.Max_Y)

  val speedX = xBounce.toggle(speed.x, -speed.x)
  val speedY = yBounce.toggle(speed.y, -speed.y)

  val pointsPlayer = rightWall.iterate(0)(_ + 1)
  val pointsComputer = leftWall.iterate(0)(_ + 1)

  val score = Signal { pointsPlayer() + " : " + pointsComputer() }
}

object Racket {
  val Height = 80
  val Width = 10
}

class Racket(val xPos: Int, val yPos: Signal[Int]) {
  val boundedYPos = Signal {
    math.min(Pong.Max_Y - Racket.Height / 2, math.max(Racket.Height / 2, yPos()))
  }

  val bounds = Signal {
    new Rectangle(xPos - Racket.Width / 2,
      boundedYPos() - Racket.Height / 2,
      Racket.Width,
      Racket.Height)
  }
}